#pragma once

#include "IInstallController.h"
#include "IPrepareModule.h"
#include "IInstallModule.h"
#include "NPBaseLib/NPThreadPool.h"
#include "AgentDLL.h"
#include <list>

class ScheduleModuleWnd;

typedef std::list<IPrepareModule *> PrepareModuleQueue;
typedef std::list<IInstallModule *> InstallModuleQueue;

struct FindPrepareModuleParameter
{
	TCHAR *guid;
	PrepareModuleQueue::iterator it;

	FindPrepareModuleParameter()
	{
		guid = NULL;
	}
};

struct FindInstallModuleParameter
{
	TCHAR *guid;
	InstallModuleQueue::iterator it;

	FindInstallModuleParameter()
	{
		guid = NULL;
	}
};

class InstallController : public IInstallController
{
public:
	InstallController();
	~InstallController();

	BOOL Install( InstallProgramInformationVector infoArray, 
				  IInstallControllerCallback *InstallControllerCallback );
	BOOL Remove( InstallProgramInformationVector infoArray, 
				IInstallControllerCallback *InstallControllerCallback );
	BOOL SetPending( const TCHAR *guid, BOOL flag );
	BOOL Cancel(const TCHAR *guid);
	BOOL Create();

	CALLBACK_RETURN OnProgress( const TCHAR *guid,
		CALLBACK_PROGRESS_SIZE currnetSize,
		CALLBACK_PROGRESS_SIZE maxSize,
		CALLBACK_PROGRESS_STATUS status );

	CALLBACK_RETURN OnPrepareFinished( const TCHAR *guid, CALLBACK_FINISHED_STATUS status, ERRORINFO *info );
	CALLBACK_RETURN OnFinished( const TCHAR *guid, CALLBACK_FINISHED_STATUS status, ERRORINFO *info );
	CALLBACK_RETURN OnPrepareStart( const TCHAR *guid );
	CALLBACK_RETURN OnInstallStart( const TCHAR *guid );

	BOOL ProcessPrepare();
	BOOL ProcessInstall();

private:
	BOOL MakePrepareModuleQueue(InstallProgramInformationVector infoVector);
	NAVERAPP_INSTALL_STATUS GetInstalledStatus(const TCHAR *guid);
	BOOL CheckLauncher();
	BOOL KillLauncher();
	BOOL StartLauncher();
	BOOL CancelModule( const TCHAR *guid );
	BOOL CancelPrepareModule(const TCHAR * guid );
	BOOL CancelInstallModule(const TCHAR * guid );
	BOOL PreparePending( const TCHAR *guid, BOOL flag );
	BOOL InstallPending( const TCHAR *guid, BOOL flag );
	BOOL NotifyInstallSchedule();
	BOOL NotifyPrepareSchedule();
	IPrepareModule *CreateNewPrepareModule( ProgramInformation programInformation );
	IInstallModule *GetCurrentInstallModule();
	IInstallModule *GetNextInstallModule();
	IPrepareModule *GetCurrentPrepareModule();
	IPrepareModule *GetNextPrepareModule();

	FindPrepareModuleParameter FindPrepareModule(const TCHAR *guid);
	FindInstallModuleParameter FindInstallModule(const TCHAR *guid);

	BOOL CancelPrepareModule(FindPrepareModuleParameter findInfo );
	BOOL CancelInstallModule(FindInstallModuleParameter findInfo );


private:
	HANDLE							m_prepareThreadPool;
	HANDLE							m_installThreadPool;
	ScheduleModuleWnd*				m_scheduleModuleWnd;
	IInstallControllerCallback*		m_installcontrollercallback;
	PrepareModuleQueue				m_prepareModuleQueue;
	InstallModuleQueue				m_installModuleQueue;
	IPrepareModule*					m_currentPrepareModule;
	IInstallModule*					m_currentInstallModule;
	BOOL							m_bLauncherStart;
};